/*
    Zelda Battle Of Hyrule
    Copyright (C) 2009  Vincent Bourgogne
    
    Origine des sources :
    Zelda Return of the Hylian
    Copyright (C) 2005-2008  Vincent Jouillat

    Please send bugreports with examples or suggestions to www.zeldaroth.fr

*/

#ifndef __JEU_H__
#define __JEU_H__

#include <SDL/SDL.h>

#include "Audio.h"
#include "Statut.h"
#include "Objet.h"
#include "Snipe.h"
#include "Joueur.h"
#include "Monde.h"
#include "Texte.h"
#include "Menu.h"
#include "Projectile.h"
#include "Generique.h"
#include "Serveur.h"
#include "Client.h"
#include "Utile.h"
#include "Partie.h"
#include "Message.h"


class Keyboard;

class Jeu {
    public :
        Jeu(Audio* audio);
        ~Jeu();
        bool init(int Type);
        void retourMenu();
        void draw(SDL_Surface* screen);
        Joueur* getJoueur();
        Joueur* getAutreJoueur(int i);
        Monde* getMonde();
        Audio* getAudio();
        Texte* getTexte();
        Statut* getStatut();
        Generique* getGenerique();
        Keyboard* getKeyboard();
        Partie* getPartie(); 
        int getZone();
        void setZone(int i);
        int getPhg(int i);
        void setPhg(int i, int j);
        bool getStop();
        bool getText();
        bool getTransition();
        
        bool estServeur();
        int getIP(int i);
        int getPortTCP();
        int getPortUDP();
        string getIPName();
        void setIP(int a, int i);
        int getNbAutreJoueur();
        bool verifZoneJoueur(int x, int y, int w, int h, int nbdir); // regarde si Link peut y aller
        bool testEpee(int x, int y, int w, int h, int type, int force, Direction direction); //blesse monstres
        bool testDegat(int x, int y, int w, int h, int type, int force, Direction direction); //blesse Link
        bool getMenu();
        void setMenu(bool b);
        void setStop(bool b);
        void setTransition(bool b);
        void setText(bool b);
        void ecrit(int id, bool cadre=true, bool anim=true, int x=8, int y=152, int w=304, int h=80);
        void affiche(SDL_Surface* gpScreen, std::string s, int x, int y);
        bool isDonjon();
        bool isDehors();
        bool isGrotte();
        void ajouteProjectile(int type, Direction dir, int a, int b, int v);
        void ajouteProjectile(int type, Direction dir, int a, int b, int v, int tireur);
        void ajouteProjectileServ(int type, Direction dir, int a, int b, int v, int tireur);
        void ajouteObjet(int type, int i, int j, int num=0, int p=0);
        void ajouteSnipe(int type, double i, double j, double dirx, double diry);
        void ajoutePiege(int type, int i, int j, bool p=true);
        bool isNotFull(int x, int y, int w, int h);
        bool isNotFull(int x, int y, int w, int h, int vol, int nage, int spectre);
        bool isWater(int x, int y, int w, int h);
        bool isVide(int x, int y, int w, int h);
        bool isGlace(int x, int y, int w, int h);
        bool isRamassable(int x, int y, int g);
        void ramasse();
        int enleve();
        void testFin();
        void videListes();
        void setNewZone(int nz);
        void setNewX(int nx);
        void setNewY(int ny);
        void finZone();
        SDL_Surface* getImageObjets();
        void lire();
        bool pousse(int i, int j, Direction dir);
        bool isMarchable(int i, int j, int vol, int nage, int spectre);
        bool finTexte(int id);
        void trouve(int id);
        bool isLink(int x, int y, int w, int h);
        int getVueHorz();
        int getVueVert();
        void setVueHorz(int i);
        void setVueVert(int i);
        void replace(); //replace les ennemis, pnj, etc.
        void setKeyboard(Keyboard* kb);
        void setGenerique(Generique* gen);
        void setMessage(int T, int J1=0, int J2=0);
        Message* getMessage();
        int dir2Int(Direction d);
        Direction int2Dir(int a);
        bool tueJoueur(int i, int j, int anim, int numJoueur, Direction direction);
        bool tueJoueurAvecFleche(int i, int j, int h, int w, int numJoueur);
        Utile fct;
        void finDePartie();
    private :
        void testRamasse(int x, int y, int w, int h);
        bool renvoieProj(int i, int j, Direction dir);
        bool coupeRideau(int i, int j);
        void scrolling();
        void ouvrePorteMaison(int x, int y);
        void ouvrePorteDonjon(int x, int y);
        void testInterrupteur(int x, int y);
        bool isDangereux(int x,int y);
        
        Audio* gpAudio;
        Generique* gpGenerique;
        Keyboard* gpKeyboard;
        int lastZone;
        int zone;
        bool stop;
        bool transition;
        bool menu;
        bool text;
        int vueHorz;
        int vueVert;
        int phg[2];
        int newzone; //lors d'un changement de zone
        int newx;
        int newy;
        Monde* gpMonde;
        Joueur* gpJoueur;
        Texte* gpTexte;
        Menu* gpMenu;
        Statut* gpStatut;
        Projectile* gpProjectile;
        Objet* gpObjet;
        Message* gpMessage;
        Snipe* gpSnipe;
        SDL_Surface* imageObjets;
        int gpType; //  1=Serveur 2=Client
        
        int nbAutreJoueur;
        Joueur** gpAutreJoueur;
        
        Serveur* gpServeur;
        Client* gpClient;
        Partie* gpPartie;
        
        int gpIP[4];
        int portTCP;
        int portUDP;
        
};

#endif  // Jeu.h
